/*
 *  very simple syslog logger
 *	for reciving remote logs and nothing more
-*
 *
 *	Jacob Dahl Pind
*/
#include "packrat.h"

/*
 * Parse command-line arguments
 */
void parse_args(state *st, int argc, char *argv[])
{
        int opt;
        /* Parse args */
        while ((opt = getopt(argc, argv, " l:a:h?-")) != -1) {
            switch (opt) {
            case 'n':
                break;
            case 'a':
            	 sstrlcpy(st->server,optarg); 
                break; 	
            case 'l':
            	 sstrlcpy(st->log_file,optarg); 
                break;
            case 'h':
            default: /* '?' */
            	usage(argv);
                exit(EXIT_FAILURE);
            }
        }
        if (optind == 1) {
        		usage(argv);
                exit(EXIT_FAILURE);

        } 
}

void usage(char *argv[])
{
	printf("Usage: %s [-l logfile] [-a ip to listen on]\n",
            argv[0]);
	return;

}

/*
 * Initialize state struct to default/empty values
 */
void init_state(state *st)
{
	  strclear(st->log_file);
	  sstrlcpy(st->username,USERNAME);
	  strclear(st->server);
}
/*
 *	signal handling
 */

void signal_handler(int signum)
{
	switch(signum) {
	case SIGTERM: 
	case SIGINT:
		unlink(PID_FILE);
		exit(EXIT_SUCCESS);
		break;
	}
}

void pid_file(pid_t pid)
{
	struct stat file;
	FILE *fp;
	char  buf[8];
	if (stat(PID_FILE_PATH,&file) == OK && (file.st_mode & S_IFDIR)){
		if ((fp = fopen(PID_FILE, "w")) != NULL){
			snprintf(buf,sizeof(buf),"%i\n",pid);
			if(fputs(buf,fp) == 0){
				syslog(LOG_ERR,"unable to write pid to %s/%s not found \n",PID_FILE_PATH,PID_FILE); exit(ERROR);
				}
			fclose(fp);
		} else {
			syslog(LOG_ERR,"unable to create %s\n",PID_FILE); exit(ERROR);
		}
	} else {
		syslog(LOG_ERR,"pid file path %s not found \n",PID_FILE_PATH); exit(ERROR);
	}

}

/*
 * Main
 */
int main(int argc, char *argv[])
{
	  state st;
	  int sd, rc, n;
	  struct sockaddr_in cliAddr, servAddr;
	  char msg[MAX_MSG];
	  FILE *fp;
	  struct tm *ltime;
	  char timestr[64];
	  time_t now;
	  struct passwd *pwd = NULL;
	  struct hostent *ph;

	 pid_t	pid,sid;
    
	 signal(SIGTERM,signal_handler);

	 /* Initialize state */
	  init_state(&st);

	  /* Handle command line arguments */
	  parse_args(&st, argc, argv);

	  pwd = getpwnam(st.username);
	  if (pwd == NULL) {
	  	printf("user not found: %s\n", st.username);
	  	exit(EXIT_FAILURE);
	  }

	  /* socket creation */
	  sd=socket(AF_INET, SOCK_DGRAM, 0);
	  if(sd<0) {
	    printf("%s: cannot open socket \n",argv[0]);
	    exit(EXIT_FAILURE);
	  }

	  ph = gethostbyname(st.server);

	  /* bind local server port */
	if (!st.server) {
	 	servAddr.sin_family = ph->h_addrtype;
	  	servAddr.sin_addr  = incopy(ph -> h_addr_list[0]);
	  	servAddr.sin_port = htons(LOCAL_SERVER_PORT);
	} else {
		servAddr.sin_family = AF_INET;
		servAddr.sin_addr.s_addr = htonl(INADDR_ANY);
		servAddr.sin_port = htons(LOCAL_SERVER_PORT);
	}

	  rc = bind (sd, (struct sockaddr *) &servAddr,sizeof(servAddr));
	  if(rc<0) {
	    printf("%s: cannot bind port number %d \n", 
		   argv[0], LOCAL_SERVER_PORT);
	    exit(EXIT_FAILURE);
	  }

	  	/* Try to open the logfile for appending */
	
	if ((fp = fopen(st.log_file, "a")) == NULL) { 
		printf("unable to open %s\n",st.log_file); exit(ERROR);
	}

	  /* detatch from shell */

	  pid=fork();
	  if (pid<0) exit(EXIT_FAILURE);
	  if (pid>0) exit(EXIT_SUCCESS);

	  sid = setsid();
	   if (sid < 0) 
        exit(EXIT_FAILURE);
       
       if ((chdir("/")) < 0) 
        exit(EXIT_FAILURE);

	  /* dropping privileges */
	  if (initgroups(st.username, pwd->pw_gid) < 0 \
	   || setgid(pwd->pw_gid) < 0 \
	   || setuid(pwd->pw_uid) < 0)
	          syslog(LOG_ERR, "unable setuid to  %s\n",st.username);

	/* Redirect standard files to /dev/null */
    freopen( "/dev/null", "r", stdin);
    freopen( "/dev/null", "w", stdout);
    freopen( "/dev/null", "w", stderr);
	
	  /*
 	   * save our pid to file
	  */
		pid_file(sid);	
	
	    /* server infinite loop */
	    while(1) {
      
	      /* init buffer */
	      memset(msg,0x0,MAX_MSG);

	      /* receive message */
	       socklen_t  cliLen = sizeof(cliAddr);
	      n = recvfrom(sd, msg, MAX_MSG, 0, 
	  		 (struct sockaddr *) &cliAddr, &cliLen);

	      if(n<0) {
	        syslog(LOG_ERR,"cannot receive data \n");
	        continue;
	      }
	       	/* Format time */
			now = time(NULL);
			ltime = localtime(&now);
		strftime(timestr,sizeof(timestr), "%b %d %H:%M:%S",ltime);
	      /* print received message */
	    if (fprintf(fp, "%s %s %s\n",timestr,inet_ntoa(cliAddr.sin_addr),msg) > 0) {
		  fflush(fp);
		} else {

		}

		    }/* end of server infinite loop */
		fclose(fp);
	  return 0;

}
